﻿using LuxSciWebApiWrapper.Exceptions;
using LuxSciWebApiWrapper.LuxApiHttpClient;
using LuxSciWebApiWrapper.Models;
using Newtonsoft.Json;
using System.Configuration;
using System.Threading.Tasks;

namespace LuxSciWebApiWrapper
{
    public class AccountApiWrapper
    {
        private string accountRequestPath = string.Empty;
        private string accountId = string.Empty;

        public AccountApiWrapper()
        {
            accountRequestPath = ConfigurationManager.AppSettings["AccountPath"];
            accountId = ConfigurationManager.AppSettings["AccountId"];
        }

        //Get Account Profile
        public async Task<string> GetAccountProfile()
        {
            //Get Authentiction code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAccountAuthenticationToken();

            LuxSciResponse luxSciResponse = null;
            //build path and signature
            string path = accountRequestPath + accountId + "/users";
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nGET\n" + path + "\n\n\n");

            //send data to accountHttpClient
            using (AccountHttpClient accountHttpClient = new AccountHttpClient())
            {
                luxSciResponse = await accountHttpClient.GetProfile(path, authCode, signature);
            }

            //return data, if success. else, throw exception
            if (luxSciResponse.success == 1)
            {
                return luxSciResponse.data.ToString();
            }
            else
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
        }

        public async Task<bool> CreateAlias(AliasRequest alias)
        {
            //Get authentication code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAccountAuthenticationToken();
            LuxSciResponse luxSciResponse = null;

            //build path and signature
            string path = accountRequestPath +  accountId + "/aliases";
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nPOST\n" + path + "\n\n" + AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject(alias)) + "\n");

            //call httpclient
            using (AccountHttpClient accountHttpClient = new AccountHttpClient())
            {
                luxSciResponse = await accountHttpClient.CreateAlias(path, authCode, signature, alias);
            }

            //throw error, if the success is not 1
            if (luxSciResponse.success != 1)
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
            else
            {
                return true;
            }
        }

        public async Task<string> GetMailSentReport(EmailType emailType)
        {
            //Get Authentication code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAccountAuthenticationToken();

            //build path and signature
            LuxSciResponse luxSciResponse = null;
            string path = string.Empty;
            if (emailType == EmailType.SMTP)
            {
                path= accountRequestPath + accountId + "/report/email/smtp/sent";
            }
            else
            {
                path = accountRequestPath + accountId + "/report/email/webmail/sent";
            }
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nGET\n" + path + "\n\n\n");

            //call httpclient
            using (AccountHttpClient accountHttpClient = new AccountHttpClient())
            {
                luxSciResponse = await accountHttpClient.GetEmailSentReport(path, authCode, signature);
            }

            //throw exception, if success is not 1
            if (luxSciResponse.success != 1)
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
            else
            {
                return luxSciResponse.data.ToString();
            }
        }


    }

   
}
