﻿using LuxSciWebApiWrapper.Models;
using System;
using System.Configuration;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace LuxSciWebApiWrapper.LuxApiHttpClient
{
    //LuxApiHttpClient is a base class which has all required common functions.
    public class LuxApiHttpClient : IDisposable
    {
        //members
        protected HttpClient client = null;
        protected string hostName = string.Empty;
        protected string requestPath = string.Empty;

        public LuxApiHttpClient()
        {
            //initialize host
            hostName = ConfigurationManager.AppSettings["Host"];
        }

        //Get HttpClient
        protected HttpClient GetHTTPClient(HttpClientHandler clientHandler)
        {
            if (client == null)
            {
                client = new HttpClient(clientHandler) { BaseAddress = new Uri($"https://{hostName}") };
                client.DefaultRequestHeaders.Add("X-Version", "1");
                client.DefaultRequestHeaders.Accept.Clear();
                client.DefaultRequestHeaders.Accept.Add(
                    new System.Net.Http.Headers.MediaTypeWithQualityHeaderValue("application/json"));
            }

            return client;
        }

        //Get HttpClientHandler
        protected HttpClientHandler GetHttpClientHandler(string authenticationCode, string hmacSignature)
        {
            HttpClientHandler httpClientHandler = new HttpClientHandler()
            {
                AllowAutoRedirect = true,
                UseCookies = true,
                CookieContainer = new CookieContainer()
            };

            Cookie clientCookie = new Cookie("signature", authenticationCode + ":" + hmacSignature);
            clientCookie.Expires = DateTime.Now.AddDays(2);
            clientCookie.Domain = hostName;
            clientCookie.Path = "/";

            httpClientHandler.CookieContainer.Add(clientCookie);

            return httpClientHandler;

        }

        //Post call
        public async Task<LuxSciResponse> PostAsJsonAsync(Object obj)
        {
            var response = await client.PostAsJsonAsync(requestPath, obj);
            return await response.Content.ReadAsAsync<LuxSciResponse>();
        }

        //Get call
        public async Task<LuxSciResponse> GetDataAsync()
        {
            var response = await client.GetAsync(requestPath);
            return await response.Content.ReadAsAsync<LuxSciResponse>();
        }

        //Send call
        public async Task<LuxSciResponse> SendAsync(HttpRequestMessage request)
        {
            var response = await client.SendAsync(request, HttpCompletionOption.ResponseContentRead);
            return await response.Content.ReadAsAsync<LuxSciResponse>();
        }

        //dispose
        public void Dispose() => client.Dispose();
    }
}
